﻿define("epi-cms/widget/HyperLinkSelector", [
//dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/Deferred",

    "dojo/dom-style",
    "dojo/dom-construct",
    "dojo/when",
    "dojo/DeferredList",
//dijit
    "dijit/_Widget",
//epi
    "epi/string",
    "epi/shell/widget/_ModelBindingMixin",
    "epi-cms/widget/_HyperLinkFieldItem",
    "epi-cms/widget/viewmodel/HyperLinkSelectorViewModel"
],
function (
//dojo
    array,
    declare,
    lang,
    Deferred,

    domStyle,
    domConstruct,
    when,
    DeferredList,
//dijit
    _Widget,
//epi
    epiString,
    _ModelBindingMixin,
    _HyperLinkFieldItem,
    HyperLinkSelectorViewModel
    ) {
    return declare([_Widget, _ModelBindingMixin], {
        // module:
        //		epi-cms/widget/HyperLinkSelector
        // summary:
        //    Represents the selector for all hyper link, for example: page, media, 3rd party providers.
        // tags:
        //    public

        value: null,

        _dfCreateWidget: null,

        modelClassName: HyperLinkSelectorViewModel,

        modelBindingMap: {
            "wrapperSettings": ["wrapperSettings"],
            "selectedWrapper": ["selectedWrapper"]
        },

        postMixInProperties: function () {
            this.inherited(arguments);

            if (!this.model && this.modelClassName) {
                var modelClass = declare(this.modelClassName);
                this.model = new modelClass({ providers: this.providers });
            }
        },

        _setSelectedWrapperAttr: function (target) {
            // summary:
            //    Disabled all wrappers, enabled selected.
            // tags:
            //    private
            if (!target) {
                return;
            }

            array.forEach(this.wrappers, function (wrapper) {
                wrapper.set("selected", false);
                if (wrapper.inputWidget) {
                    wrapper.inputWidget.set("state", "");
                }
            });

            if (target.inputWidget) {
                target.set("selected", true); // mark this widget as selected/enabled
            }
        },

        _setWrapperSettingsAttr: function (settings) {
            var wrappers = [];
            array.forEach(settings, lang.hitch(this, function (setting) {
                setting.groupname += this.id;
                var wrapper = new _HyperLinkFieldItem(setting);

                if (setting.invisible) {
                    domStyle.set(wrapper.domNode, { display: "none" });
                }

                wrapper.on("radioButtonChange", lang.hitch(this, function () {
                    this.set("selectedWrapper", wrapper);
                    wrapper.focus();
                }));
                wrappers.push(wrapper);
            }));

            this.set("wrappers", wrappers);
        },

        _setWrappersAttr: function (wrappers) {
            this._set("wrappers", wrappers);

            // Reset deferred list
            if (this._dfCreateWidget) {
                this._dfCreateWidget.cancel();
                this._dfCreateWidget = null;
            }

            var deferredList = [],
                addChild = lang.hitch(this, function (ctor, wrapper) {
                var child = new ctor(wrapper.settings);
                wrapper.addChild(child);
                // set required after creating wrapper's child.
                wrapper.set("required", this.required);
            });

            array.forEach(wrappers, function (wrapper) {
                var df = new Deferred();
                deferredList.push(df);

                var ctor = lang.getObject(wrapper.widgetType);
                if (!ctor) {
                    require([epiString.slashName(wrapper.widgetType)], function (ctor) {
                        addChild(ctor, wrapper);
                        df.resolve();
                    });
                } else {
                    addChild(ctor, wrapper);
                    df.resolve();
                }
            });

            this._dfCreateWidget = new DeferredList(deferredList);

            when(this._dfCreateWidget, lang.hitch(this, function (results) {
                array.forEach(wrappers, function (wrapper) {
                    domConstruct.place(wrapper.domNode, this.domNode);
                }, this);

                this.onSelectorsCreated(this);

                if (this.value) { // make sure the created widget has value
                    this._setValueForWidgets(this.value);
                } else if (this.wrappers && this.wrappers.length > 0) {
                    // if hasn't value then set the first wrapper as default selected
                    this.wrappers[0].set("selected", true);
                }
            }));
        },

        onSelectorsCreated: function (widget) {
            // summary:
            //    Raise when finish constructing hyperlinkSelector.
            // tags:
            //    public
        },

        _getSelectedWrapper: function () {
            // summary:
            //    Get selected widget.
            // tags:
            //    private
            return array.filter(this.wrappers, function (wrapper) {
                return (wrapper.inputWidget && wrapper.get("selected"));
            })[0];
        },

        _getValueAttr: function () {
            // summary:
            //    Get selected href.
            // tags:
            //    private
            var selectedWrapper = this._getSelectedWrapper();
            if (selectedWrapper) {
                if (selectedWrapper.inputWidget && selectedWrapper.settings.isContentLink) {
                    this.value = selectedWrapper.inputWidget.get("permanentLink");
                } else
                    this.value = selectedWrapper.get("value");
            }
            return this.value;
        },

        _setValueAttr: function (value) {
            // summary:
            //    Set selected href.
            // tags:
            //    private
            this._set("value", value);
            this._setValueForWidgets(value);
        },

        _setValueForWidgets: function (value) {
            // Reset wrappers value to default
            array.forEach(this.wrappers, function (wrapper) {
                wrapper.set("value", null);
            });

            if (!value) {
                return;
            }

            when(this.model.getContentData(value), lang.hitch(this, function (content) {
                if (content) { // dynamic link (pages, media, products)
                    array.every(this.wrappers, function (wrapper) {
                        if (wrapper && wrapper.settings.isContentLink) {
                            if (array.some(wrapper.settings.allowedTypes, lang.hitch(this, function (baseType) {
                                return this.model.isBaseTypeIdentifier(content.typeIdentifier, baseType);
                            }))) {
                                wrapper.set("value", content.contentLink);

                                var showAllLanguages = content.missingLanguageBranch && content.missingLanguageBranch.isTranslationNeeded;
                                wrapper.set("showAllLanguages", showAllLanguages);

                                this.set("selectedWrapper", wrapper);
                                return false; // break the loop
                            }
                        }
                        return true; // continue the loop
                    }, this);
                } else { // static link (email, external links)
                    array.every(this.wrappers, function (wrapper) {
                        if (wrapper && !wrapper.settings.isContentLink) {
                            var widget = wrapper.inputWidget;
                            if (widget && widget.validator(value, widget.constraints)) {
                                wrapper.set("value", value);
                                this.set("selectedWrapper", wrapper);
                                return false; // break the loop
                            }
                        }
                        return true; // continue the loop
                    }, this);
                }
            }));
        },

        validate: function () {
            // summary:
            //    Validation value of Href.
            // tags:
            //    public
            var selectedWrapper = this._getSelectedWrapper();
            return selectedWrapper ? selectedWrapper.validate() : false;
        },

        focus: function () {
            // summary:
            //    TODOC
            // tags:
            //    public
            var selectedWrapper = this._getSelectedWrapper();

            if (selectedWrapper) {
                selectedWrapper.focus();
            }
        },

        reset: function () {
            this.set("value", null);
        }
    });
});
