﻿define("epi/shell/command/OptionCommand", [
    "dojo/_base/declare",
    "epi/shell/command/_Command"
], function (declare, _Command) {

    // module:
    //		epi/shell/command/OptionCommand
    // summary:
    //		A base class implementation for a command with several options that execute when changed.

    return declare([_Command], {

        // options: [public] Array
        //		An array of options objects that can be selected.
        options: null,

        // selected: [readonly] String|Number
        //		The value of the selected item from the options array.
        selected: null,

        // property: [public] String
        //		The name of the property on the model that will be set with the selected value on execute.
        property: null,

        postscript: function () {
            // summary:
            //		Watch model and initialize model dependent properties.
            // tags:
            //		public

            this.inherited(arguments);

            this.watch("selected", function (name, oldValue, newValue) {
                if (oldValue !== newValue) {
                    this.execute();
                }
            });
        },

        _execute: function () {
            // summary:
            //		Toggles the value of the given property on the model.
            // tags:
            //		protected

            this.model.set(this.property, this.selected);
        },

        _onModelChange: function () {
            // summary:
            //		Updates canExecute after the model has been updated.
            // tags:
            //		protected

            var model = this.model;

            this.set("canExecute", !!model);
            this.set("selected", model && model[this.property]);
        }
    });
});